(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('redux-resource')) :
	typeof define === 'function' && define.amd ? define(['exports', 'redux-resource'], factory) :
	(factory((global.ReduxResourcePlugins = {}),global.ReduxResource));
}(this, (function (exports,reduxResource) { 'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};



















var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



































var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

// End actions can be failed, succeeded, or null. Null should be dispatched
// when the request is aborted (with a status code of 0).
var createEndActions = [reduxResource.actionTypes.CREATE_RESOURCES_FAILED, reduxResource.actionTypes.CREATE_RESOURCES_SUCCEEDED, reduxResource.actionTypes.CREATE_RESOURCES_NULL];

var readEndActions = [reduxResource.actionTypes.READ_RESOURCES_FAILED, reduxResource.actionTypes.READ_RESOURCES_SUCCEEDED, reduxResource.actionTypes.READ_RESOURCES_NULL];

var updateEndActions = [reduxResource.actionTypes.UPDATE_RESOURCES_FAILED, reduxResource.actionTypes.UPDATE_RESOURCES_SUCCEEDED, reduxResource.actionTypes.UPDATE_RESOURCES_NULL];

var deleteEndActions = [reduxResource.actionTypes.DELETE_RESOURCES_FAILED, reduxResource.actionTypes.DELETE_RESOURCES_SUCCEEDED, reduxResource.actionTypes.DELETE_RESOURCES_NULL];

// This sets a new meta property on resource and request metadata: `statusCode`.
// This will be equal to the last status code for a request
function httpStatusCodes(resourceName) {
  return function (state, action) {
    if (action.resourceName !== resourceName) {
      return state;
    }

    var isCreateEndAction = createEndActions.indexOf(action.type) !== -1;
    var isReadEndAction = readEndActions.indexOf(action.type) !== -1;
    var isUpdateEndAction = updateEndActions.indexOf(action.type) !== -1;
    var isDeleteEndAction = deleteEndActions.indexOf(action.type) !== -1;

    if (!isCreateEndAction && !isReadEndAction && !isUpdateEndAction && !isDeleteEndAction) {
      return state;
    }

    // If we have no statusCode, we still want to set the value to 0. Browsers
    // tend to use 0 as a "null" state (i.e.; that is the status of a request
    // that has not yet completed).
    var statusCode = action.statusCode || 0;
    var resources = action.resources;

    var request = void 0;
    if (action.request && typeof action.request === 'string') {
      request = action.request;
    }

    var newRequests = void 0,
        newMeta = void 0,
        idList = void 0;
    if (resources) {
      idList = resources.map(function (r) {
        if ((typeof r === 'undefined' ? 'undefined' : _typeof(r)) === 'object') {
          return r.id;
        } else {
          return r;
        }
      });
    } else {
      idList = [];
    }

    if (request) {
      var existingRequest = state.requests[request] || {};

      newRequests = _extends({}, state.requests, defineProperty({}, request, _extends({}, existingRequest, {
        statusCode: statusCode
      })));
    } else {
      newRequests = _extends({}, state.requests);
    }

    if (idList.length) {
      var metaPrefix = void 0;
      if (isCreateEndAction) {
        metaPrefix = 'create';
      } else if (isReadEndAction) {
        metaPrefix = 'read';
      } else if (isUpdateEndAction) {
        metaPrefix = 'update';
      } else if (isDeleteEndAction) {
        metaPrefix = 'delete';
      }

      newMeta = reduxResource.setResourceMeta({
        meta: state.meta,
        newMeta: defineProperty({}, metaPrefix + 'StatusCode', statusCode),
        resources: idList,
        mergeMeta: true
      });
    } else {
      newMeta = state.meta;
    }

    return _extends({}, state, {
      requests: newRequests,
      meta: newMeta
    });
  };
}

// This plugin adds support for "compound documents," or including multiple
// resource types in a single action.
function includedResources(resourceName) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  return function (state, action) {
    var initialResourceMeta = options.initialResourceMeta;
    var includedResources = action.includedResources,
        mergeMeta = action.mergeMeta,
        mergeResources = action.mergeResources,
        type = action.type;

    // This plugin only works for successful reads at the moment

    if (type !== reduxResource.actionTypes.READ_RESOURCES_SUCCEEDED) {
      return state;
    }

    // If this action has no includedResources, then there is nothing to do
    if (!includedResources) {
      return state;
    }

    // If there are no included resources for this slice, then we do nothing
    var includedResourceList = includedResources[resourceName];

    if (!includedResourceList) {
      return state;
    }

    var resources = reduxResource.upsertResources(state.resources, includedResourceList, mergeResources);

    var meta = reduxResource.setResourceMeta({
      resources: includedResourceList,
      meta: state.meta,
      newMeta: {
        readStatus: reduxResource.requestStatuses.SUCCEEDED
      },
      initialResourceMeta: initialResourceMeta,
      mergeMeta: mergeMeta
    });

    return _extends({}, state, {
      meta: meta,
      resources: resources
    });
  };
}

var actionTypes$1 = {
  SELECT_RESOURCES: 'SELECT_RESOURCES',
  DESELECT_RESOURCES: 'DESELECT_RESOURCES',
  CLEAR_SELECTED_RESOURCES: 'CLEAR_SELECTED_RESOURCES'
};

var initialState = {
  selectedIds: []
};

function selectResources(resourceName, resources) {
  return {
    type: actionTypes$1.SELECT_RESOURCES,
    resourceName: resourceName,
    resources: resources
  };
}

function deselectResources(resourceName, resources) {
  return {
    type: actionTypes$1.DESELECT_RESOURCES,
    resourceName: resourceName,
    resources: resources
  };
}

function clearSelectedResources(resourceName) {
  return {
    type: actionTypes$1.CLEAR_SELECTED_RESOURCES,
    resourceName: resourceName
  };
}

function selection(resourceName) {
  return function (state, action) {
    // Ignore actions that were dispatched for another resource type
    if (action.resourceName !== resourceName) {
      return state;
    }

    var selectedIds = void 0,
        resourceIds = void 0;
    var resources = action.resources || [];

    if (action.type === actionTypes$1.SELECT_RESOURCES) {
      selectedIds = [].concat(toConsumableArray(state.selectedIds)) || [];
      resourceIds = resources.map(function (r) {
        return (typeof r === 'undefined' ? 'undefined' : _typeof(r)) === 'object' ? r.id : r;
      });

      resourceIds.forEach(function (id) {
        if (!selectedIds.includes(id)) {
          selectedIds.push(id);
        }
      });

      return _extends({}, state, {
        selectedIds: selectedIds
      });
    } else if (action.type === actionTypes$1.DESELECT_RESOURCES) {
      selectedIds = [].concat(toConsumableArray(state.selectedIds)) || [];
      resourceIds = resources.map(function (r) {
        return (typeof r === 'undefined' ? 'undefined' : _typeof(r)) === 'object' ? r.id : r;
      });

      return _extends({}, state, {
        selectedIds: selectedIds.filter(function (id) {
          return !resourceIds.includes(id);
        })
      });
    } else if (action.type === actionTypes$1.CLEAR_SELECTED_RESOURCES) {
      return _extends({}, state, {
        selectedIds: []
      });
    }

    return state;
  };
}

selection.actionTypes = actionTypes$1;
selection.selectResources = selectResources;
selection.deselectResources = deselectResources;
selection.clearSelectedResources = clearSelectedResources;
selection.initialState = initialState;

var actionTypes$2 = {
  RESET_RESOURCE: 'RESET_RESOURCE'
};

function resetResource(resourceName) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      request = _ref.request,
      list = _ref.list;

  return {
    type: 'RESET_RESOURCE',
    resourceName: resourceName,
    request: request,
    list: list
  };
}

function reset(resourceName) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  return function (state, action) {
    if (action.type !== actionTypes$2.RESET_RESOURCE || action.resourceName !== resourceName) {
      return state;
    }

    var request = action.request,
        list = action.list;


    if (!request && !list) {
      return _extends({
        resources: {},
        meta: {},
        lists: {},
        requests: {}
      }, options.initialState);
    }

    var newState = _extends({}, state);

    if (request) {
      newState.requests = _extends({}, state.requests, defineProperty({}, request, {
        ids: [],
        status: reduxResource.requestStatuses.NULL
      }));
    }

    if (list) {
      newState.lists = _extends({}, state.lists, defineProperty({}, list, []));
    }

    return newState;
  };
}

reset.actionTypes = actionTypes$2;
reset.resetResource = resetResource;

exports.httpStatusCodes = httpStatusCodes;
exports.selection = selection;
exports.reset = reset;
exports.includedResources = includedResources;

Object.defineProperty(exports, '__esModule', { value: true });

})));
