var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import { upsertResources, setResourceMeta, requestStatuses, actionTypes } from 'redux-resource';

// This plugin adds support for "compound documents," or including multiple
// resource types in a single action.
export default function includedResources(resourceName) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  return function (state, action) {
    var initialResourceMeta = options.initialResourceMeta;
    var includedResources = action.includedResources,
        mergeMeta = action.mergeMeta,
        mergeResources = action.mergeResources,
        type = action.type;

    // This plugin only works for successful reads at the moment

    if (type !== actionTypes.READ_RESOURCES_SUCCEEDED) {
      return state;
    }

    // If this action has no includedResources, then there is nothing to do
    if (!includedResources) {
      return state;
    }

    // If there are no included resources for this slice, then we do nothing
    var includedResourceList = includedResources[resourceName];

    if (!includedResourceList) {
      return state;
    }

    var resources = upsertResources(state.resources, includedResourceList, mergeResources);

    var meta = setResourceMeta({
      resources: includedResourceList,
      meta: state.meta,
      newMeta: {
        readStatus: requestStatuses.SUCCEEDED
      },
      initialResourceMeta: initialResourceMeta,
      mergeMeta: mergeMeta
    });

    return _extends({}, state, {
      meta: meta,
      resources: resources
    });
  };
}