'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = includedResources;

var _reduxResource = require('redux-resource');

// This plugin adds support for "compound documents," or including multiple
// resource types in a single action.
function includedResources(resourceName, options = {}) {
  return (state, action) => {
    const { initialResourceMeta } = options;
    const { includedResources, mergeMeta, mergeResources, type } = action;

    // This plugin only works for successful reads at the moment
    if (type !== _reduxResource.actionTypes.READ_RESOURCES_SUCCEEDED) {
      return state;
    }

    // If this action has no includedResources, then there is nothing to do
    if (!includedResources) {
      return state;
    }

    // If there are no included resources for this slice, then we do nothing
    const includedResourceList = includedResources[resourceName];

    if (!includedResourceList) {
      return state;
    }

    const resources = (0, _reduxResource.upsertResources)(state.resources, includedResourceList, mergeResources);

    const meta = (0, _reduxResource.setResourceMeta)({
      resources: includedResourceList,
      meta: state.meta,
      newMeta: {
        readStatus: _reduxResource.requestStatuses.SUCCEEDED
      },
      initialResourceMeta,
      mergeMeta
    });

    return _extends({}, state, {
      meta,
      resources
    });
  };
}