'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reduxResource = require('redux-resource');

const actionTypes = {
  RESET_RESOURCE: 'RESET_RESOURCE'
};

function resetResource(resourceName, { request, list } = {}) {
  return {
    type: 'RESET_RESOURCE',
    resourceName,
    request,
    list
  };
}

function reset(resourceName, options = {}) {
  return function (state, action) {
    if (action.type !== actionTypes.RESET_RESOURCE || action.resourceName !== resourceName) {
      return state;
    }

    const { request, list } = action;

    if (!request && !list) {
      return _extends({
        resources: {},
        meta: {},
        lists: {},
        requests: {}
      }, options.initialState);
    }

    const newState = _extends({}, state);

    if (request) {
      newState.requests = _extends({}, state.requests, {
        [request]: {
          ids: [],
          status: _reduxResource.requestStatuses.NULL
        }
      });
    }

    if (list) {
      newState.lists = _extends({}, state.lists, {
        [list]: []
      });
    }

    return newState;
  };
}

reset.actionTypes = actionTypes;
reset.resetResource = resetResource;

exports.default = reset;